/******************************************************************************\
|* Copyright (c) 2020 by VeriSilicon Holdings Co., Ltd. ("VeriSilicon")       *|
|* All Rights Reserved.                                                       *|
|*                                                                            *|
|* The material in this file is confidential and contains trade secrets of    *|
|* of VeriSilicon.  This is proprietary information owned or licensed by      *|
|* VeriSilicon.  No part of this work may be disclosed, reproduced, copied,   *|
|* transmitted, or used in any way for any purpose, without the express       *|
|* written permission of VeriSilicon.                                         *|
|*                                                                            *|
\******************************************************************************/

#ifndef _VIV_V4L2_IOCTL_H_
#define _VIV_V4L2_IOCTL_H_

#include "cam_device_api.hpp"
#include "viv_v4l2_ioctl_ids.h"
#include <unordered_map>
#include <string>

/*  header of string, via v4l2_s_ctrl
    no need this map when using VIV_VIDIOC_CONTROL_EVENT */
static std::unordered_map<std::string, int> v4l2_cid_map = {
    { IF_AE_S_CFG,      ISPCORE_MODULE_AE_CFG_SET },
    { IF_AE_G_CFG,      ISPCORE_MODULE_AE_CFG_GET },
    { IF_AE_S_ECM,      ISPCORE_MODULE_AE_ECM_SET },
    { IF_AE_G_ECM,      ISPCORE_MODULE_AE_ECM_GET },
    { IF_AE_S_EN,       ISPCORE_MODULE_AE_ENABLE_SET },
    { IF_AE_G_EN,       ISPCORE_MODULE_AE_ENABLE_GET },
    { IF_AE_RESET,      ISPCORE_MODULE_AE_RESET },
    { IF_AE_G_STATUS,   ISPCORE_MODULE_AE_STATUS_GET },
    { IF_AE_S_ISO,      ISPCORE_MODULE_AE_SENSITIVITY_SET},
    { IF_AE_G_ISO,      ISPCORE_MODULE_AE_SENSITIVITY_GET},
    { IF_AF_S_CFG,      ISPCORE_MODULE_AF_CFG_SET },
    { IF_AF_G_CFG,      ISPCORE_MODULE_AF_CFG_GET },
    { IF_AF_S_EN,       ISPCORE_MODULE_AF_ENABLE_SET },
    { IF_AF_G_EN,       ISPCORE_MODULE_AF_ENABLE_GET },
    { IF_AF_G_AVI,      ISPCORE_MODULE_AF_AVI_GET },
    { IF_AVS_S_CFG,     ISPCORE_MODULE_AVS_CFG_SET },
    { IF_AVS_G_CFG,     ISPCORE_MODULE_AVS_CFG_GET },
    { IF_AVS_S_EN,      ISPCORE_MODULE_AVS_ENABLE_SET },
    { IF_AVS_G_EN,      ISPCORE_MODULE_AVS_ENABLE_GET },
    { IF_AWB_S_CFG,     ISPCORE_MODULE_AWB_CFG_SET },
    { IF_AWB_G_CFG,     ISPCORE_MODULE_AWB_CFG_GET },
    { IF_AWB_S_EN,      ISPCORE_MODULE_AWB_ENABLE_SET },
    { IF_AWB_G_EN,      ISPCORE_MODULE_AWB_ENABLE_GET },
    { IF_AWB_RESET,     ISPCORE_MODULE_AWB_RESET },
    { IF_AWB_G_STATUS,  ISPCORE_MODULE_AWB_STATUS_GET },
    { IF_AWB_G_ILLUMPRO,ISPCORE_MODULE_AWB_ILLUMPRO_GET },
    { IF_AWB_S_MEASWIN, ISPCORE_MODULE_AWB_MEAS_WIN_SET },
    { IF_BLS_S_CFG,     ISPCORE_MODULE_BLS_CFG_SET },
    { IF_BLS_G_CFG,     ISPCORE_MODULE_BLS_CFG_GET },
    { IF_CAC_S_EN,      ISPCORE_MODULE_CAC_ENABLE_SET },
    { IF_CAC_G_EN,      ISPCORE_MODULE_CAC_ENABLE_GET },
    { IF_CNR_S_CFG,     ISPCORE_MODULE_CNR_CFG_SET },
    { IF_CNR_G_CFG,     ISPCORE_MODULE_CNR_CFG_GET },
    { IF_CNR_S_EN,      ISPCORE_MODULE_CNR_ENABLE_SET },
    { IF_CNR_G_EN,      ISPCORE_MODULE_CNR_ENABLE_GET },
    { IF_CPROC_S_CFG,   ISPCORE_MODULE_CPROC_CFG_SET },
    { IF_CPROC_G_CFG,   ISPCORE_MODULE_CPROC_CFG_GET },
    { IF_CPROC_S_EN,    ISPCORE_MODULE_CPROC_ENABLE_SET },
    { IF_CPROC_G_EN,    ISPCORE_MODULE_CPROC_ENABLE_GET },
    { IF_CPROC_S_COEFF,     ISPCORE_MODULE_CPROC_COEFF_SET },
    { IF_DEMOSAIC_S_CFG,    ISPCORE_MODULE_DEMOSAIC_CFG_SET },
    { IF_DEMOSAIC_G_CFG,    ISPCORE_MODULE_DEMOSAIC_CFG_GET },
    { IF_DEMOSAIC_S_EN, ISPCORE_MODULE_DEMOSAIC_ENABLE_SET },
    { IF_DEMOSAIC_G_EN, ISPCORE_MODULE_DEMOSAIC_ENABLE_GET },
    { IF_2DNR_S_CFG,    ISPCORE_MODULE_2DNR_CFG_SET },
    { IF_2DNR_G_CFG,    ISPCORE_MODULE_2DNR_CFG_GET },
    { IF_2DNR_S_EN,     ISPCORE_MODULE_2DNR_ENABLE_SET },
    { IF_2DNR_G_EN,     ISPCORE_MODULE_2DNR_ENABLE_GET },
    { IF_2DNR_RESET,    ISPCORE_MODULE_2DNR_RESET },
    { IF_2DNR_S_TBL,    ISPCORE_MODULE_2DNR_TABLE_SET },
    { IF_3DNR_S_CFG,    ISPCORE_MODULE_3DNR_CFG_SET },
    { IF_3DNR_G_CFG,    ISPCORE_MODULE_3DNR_CFG_GET },
    { IF_3DNR_S_EN,     ISPCORE_MODULE_3DNR_ENABLE_SET },
    { IF_3DNR_G_EN,     ISPCORE_MODULE_3DNR_ENABLE_GET },
    { IF_3DNR_RESET,    ISPCORE_MODULE_3DNR_RESET },
    { IF_3DNR_S_TBL,    ISPCORE_MODULE_3DNR_TABLE_SET },
    { IF_DPCC_S_EN,     ISPCORE_MODULE_DPCC_ENABLE_SET },
    { IF_DPCC_G_EN,     ISPCORE_MODULE_DPCC_ENABLE_GET },
    { IF_DPF_S_CFG,     ISPCORE_MODULE_DPF_CFG_SET },
    { IF_DPF_G_CFG,     ISPCORE_MODULE_DPF_CFG_GET },
    { IF_DPF_S_EN,      ISPCORE_MODULE_DPF_ENABLE_SET },
    { IF_DPF_G_EN,      ISPCORE_MODULE_DPF_ENABLE_GET },
    { IF_EC_S_CFG,      ISPCORE_MODULE_EC_CFG_SET },
    { IF_EC_G_CFG,      ISPCORE_MODULE_EC_CFG_GET },
    { IF_EC_G_STATUS,   ISPCORE_MODULE_EC_STATUS_GET },
    { IF_EE_S_CFG,      ISPCORE_MODULE_EE_CFG_SET },
    { IF_EE_G_CFG,      ISPCORE_MODULE_EE_CFG_GET },
    { IF_EE_S_EN,       ISPCORE_MODULE_EE_ENABLE_SET },
    { IF_EE_G_EN,       ISPCORE_MODULE_EE_ENABLE_GET },
    { IF_EE_RESET,      ISPCORE_MODULE_EE_RESET },
    { IF_EE_S_TBL,      ISPCORE_MODULE_EE_TABLE_SET },
    { IF_FILTER_S_CFG,  ISPCORE_MODULE_FILTER_CFG_SET },
    { IF_FILTER_G_CFG,  ISPCORE_MODULE_FILTER_CFG_GET },
    { IF_FILTER_S_EN,   ISPCORE_MODULE_FILTER_ENABLE_SET },
    { IF_FILTER_G_EN,   ISPCORE_MODULE_FILTER_ENABLE_GET },
    { IF_FILTER_S_TBL,  ISPCORE_MODULE_FILTER_TABLE_SET },
    { IF_FILTER_G_TBL,  ISPCORE_MODULE_FILTER_TABLE_GET },
    { IF_FILTER_G_STATUS, ISPCORE_MODULE_FILTER_STATUS_GET },
    { IF_GC_S_CURVE,    ISPCORE_MODULE_GC_CURVE_SET },
    { IF_GC_G_CURVE,    ISPCORE_MODULE_GC_CURVE_GET },
    { IF_GC_S_CFG,      ISPCORE_MODULE_GC_CFG_SET },
    { IF_GC_G_CFG,      ISPCORE_MODULE_GC_CFG_GET },
    { IF_GC_S_EN,       ISPCORE_MODULE_GC_ENABLE_SET },
    { IF_GC_G_EN,       ISPCORE_MODULE_GC_ENABLE_GET },
    { IF_HDR_S_CFG,     ISPCORE_MODULE_HDR_CFG_SET },
    { IF_HDR_G_CFG,     ISPCORE_MODULE_HDR_CFG_GET },
    { IF_HDR_S_EN,      ISPCORE_MODULE_HDR_ENABLE_SET },
    { IF_HDR_G_EN,      ISPCORE_MODULE_HDR_ENABLE_GET },
    { IF_HDR_RESET,     ISPCORE_MODULE_HDR_RESET },
    { IF_IE_S_CFG,      ISPCORE_MODULE_IE_CFG_SET },
    { IF_IE_G_CFG,      ISPCORE_MODULE_IE_CFG_GET },
    { IF_IE_S_EN,       ISPCORE_MODULE_IE_ENABLE_SET },
    { IF_IE_G_EN,       ISPCORE_MODULE_IE_ENABLE_GET },
    { IF_LSC_S_CFG,     ISPCORE_MODULE_LSC_CFG_SET },
    { IF_LSC_S_EN,      ISPCORE_MODULE_LSC_ENABLE_SET },
    { IF_LSC_G_EN,      ISPCORE_MODULE_LSC_ENABLE_GET },
    { IF_LSC_G_STATUS,  ISPCORE_MODULE_LSC_STATUS_GET },
    { IF_SIMP_S_CFG,    ISPCORE_MODULE_SIMP_CFG_SET },
    { IF_SIMP_S_EN,     ISPCORE_MODULE_SIMP_ENABLE_SET },
    { IF_WB_S_CFG,      ISPCORE_MODULE_WB_CFG_SET },
    { IF_WB_G_CFG,      ISPCORE_MODULE_WB_CFG_GET },
    { IF_WB_S_GAIN,     ISPCORE_MODULE_WB_CFG_SET },
    { IF_WB_S_CCM,      ISPCORE_MODULE_WB_CFG_SET },
    { IF_WB_S_OFFSET,   ISPCORE_MODULE_WB_CFG_SET },
    { IF_WDR_S_CFG,     ISPCORE_MODULE_WDR_CFG_SET },
    { IF_WDR_G_CFG,     ISPCORE_MODULE_WDR_CFG_GET },
    { IF_WDR_S_EN,      ISPCORE_MODULE_WDR_ENABLE_SET },
    { IF_WDR_G_EN,      ISPCORE_MODULE_WDR_ENABLE_GET },
    { IF_WDR_G_STATUS,  ISPCORE_MODULE_WDR_STATUS_GET },
    { IF_WDR_RESET,     ISPCORE_MODULE_WDR_RESET },
    { IF_WDR_S_TBL,     ISPCORE_MODULE_WDR_TABLE_SET },
    { IF_WDR_G_TBL,     ISPCORE_MODULE_WDR_TABLE_GET },
    { IF_SENSOR_QUERY,  ISPCORE_MODULE_SENSOR_QUERY },
    { IF_SENSOR_G_MODE, ISPCORE_MODULE_SENSOR_MODE_GET },
    { IF_SENSOR_S_MODE, ISPCORE_MODULE_SENSOR_MODE_SET },
    { IF_SENSOR_G_RESW, ISPCORE_MODULE_SENSOR_RES_W_GET },
    { IF_SENSOR_G_RESH, ISPCORE_MODULE_SENSOR_RES_H_GET },
    { IF_SENSOR_G_REG,  ISPCORE_MODULE_SENSOR_REG_GET },
    { IF_SENSOR_S_REG,  ISPCORE_MODULE_SENSOR_REG_SET },
    { IF_SENSOR_G_SEC,  ISPCORE_MODULE_SENSOR_SEC_GET },
    { IF_SENSOR_S_SEC,  ISPCORE_MODULE_SENSOR_SEC_SET },
    { IF_SENSOR_S_TESTPAT, ISPCORE_MODULE_SENSOR_TESTPATTERN_SET },
    { IF_SENSOR_INFO,   ISPCORE_MODULE_SENSOR_INFO },

    { IF_DWE_G_PARAMS,  VIV_V4L_DWE_GET_PARAM },
    { IF_DWE_S_PARAMS,  VIV_V4L_DWE_SET_PARAM },
    { IF_DWE_G_HFLIP,   VIV_V4L_DWE_GET_HFLIP },
    { IF_DWE_S_HFLIP,   VIV_V4L_DWE_SET_HFLIP },
    { IF_DWE_G_VFLIP,   VIV_V4L_DWE_GET_VFLIP },
    { IF_DWE_S_VFLIP,   VIV_V4L_DWE_SET_VFLIP },
    { IF_DWE_G_BYPASS,  VIV_V4L_DWE_GET_BYPASS },
    { IF_DWE_S_BYPASS,  VIV_V4L_DWE_SET_BYPASS },
    { IF_DWE_G_MODE,    VIV_V4L_DWE_GET_MODE },
    { IF_DWE_S_MODE,    VIV_V4L_DWE_SET_MODE },
    { IF_DWE_G_MAT,     VIV_V4L_DWE_GET_MATRIX },
    { IF_DWE_S_MAT,     VIV_V4L_DWE_SET_MATRIX },
    { IF_DWE_G_TYPE,    VIV_V4L_DWE_GET_TYPE },
    { IF_DWE_S_TYPE,    VIV_V4L_DWE_SET_TYPE },

    { IF_CAPTURE,       VIV_V4L_MC_CAPTURE },
    { IF_S_FPS,                 ISPCORE_MODULE_SENSOR_FPS_SET },
    { IF_OUT_FMT,               ISPCORE_MODULE_DEVICE_OUTPUT_IMAGE_FORMAT },
    { IF_G_RESOLUTION,          ISPCORE_MODULE_DEVICE_RESOLUTION_GET },
    { IF_SENSOR_LIB_PRELOAD,    ISPCORE_MODULE_SENSOR_PRE_LOAD_LIB },
    { IF_PIPELINE_S_WARM_UP,    ISPCORE_MODULE_PIPELINE_WARM_UP },
    { IF_CALIBRATION_GET,       ISPCORE_MODULE_DEVICE_CALIBRATIONSAVE},
    { IF_CALIBRATION_SET,       ISPCORE_MODULE_DEVICE_CALIBRATIONLOAD},
    { IF_INPUTINFO,             ISPCORE_MODULE_DEVICE_INPUTINFO},
    { IF_INPUTSWITCH,           ISPCORE_MODULE_DEVICE_INPUTSWITCH},

    { IF_PIPELINE_S_SMP_MODE,   ISPCORE_MODULE_PIPELINE_SMP_MODE },
    { IF_PIPELINE_S_DWE_ONOFF,  ISPCORE_MODULE_PIPELINE_CFG_DWE },
    { IF_PIPELINE_S_TESTPAT,    ISPCORE_MODULE_PIPELINE_CFG_TESTPATTEN_EN },
    { IF_PIPELINE_S_RES_IS_OUT, ISPCORE_MODULE_PIPELINE_RESOLUTION_IS_OUT },
    { IF_PIPELINE_S_RES_MP_OUT, ISPCORE_MODULE_PIPELINE_RESOLUTION_MP_OUT },
    { IF_PIPELINE_S_MP_FMT,     ISPCORE_MODULE_PIPELINE_MP_OUT_IMAGE_FORMAT },
    { IF_PIPELINE_QUERY,        ISPCORE_MODULE_PIPELINE_QUERY },
    { IF_PIPELINE_CFG_STATUS,   ISPCORE_MODULE_PIPELINE_CFG_STATUS },
    { IF_PIPELINE_S_3A_LOCK,    ISPCORE_MODULE_PIPELINE_3A_LOCK_SET},
    { IF_PIPELINE_G_3A_LOCK,    ISPCORE_MODULE_PIPELINE_3A_LOCK_GET},

    { IF_MODULE_DATA_PARSE,  ISPCORE_MODULE_DEVICE_DATA_PARSE },
    { IF_MODULE_DATA_G,      ISPCORE_MODULE_DEVICE_DATA_GET },

};

#endif  // _VIV_V4L2_IOCTL_H_
